package UnitTesting;

import SQLBuilder.QueryBuilder;

import org.junit.experimental.categories.Category;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class SQLBuildingTests {

    QueryBuilder qry;

    @AfterEach
    void tearDown() {
        qry = null;
    }

    @Test
    @DisplayName("Impressions")
    public void impressionQueryTest(){
        qry = new QueryBuilder("Impressions", "Day", "");
        assertEquals(qry.getQuery(), "SELECT COUNT(id) FROM impression_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT date(impression_log.date) AS granularity, COUNT(id) FROM impression_log GROUP BY granularity");
    }

    @Test
    @DisplayName("Clicks")
    public void clicksQueryTest(){
        qry = new QueryBuilder("Clicks", "Day", "");
        assertEquals(qry.getQuery(), "SELECT COUNT(id) FROM click_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT date(date) AS granularity, COUNT(id) FROM click_log GROUP BY granularity");
        assertEquals(qry.getQuery("age groups"), "SELECT DISTINCT age AS uniq_age, COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON impression_log.user_id = click_log.user_id GROUP BY uniq_age");
    }

    @Test
    @DisplayName("Unique Users")
    public void uniqQueryTest(){
        qry = new QueryBuilder("Unique Users", "Day", "");
        assertEquals(qry.getQuery(), "SELECT COUNT(DISTINCT user_id) FROM click_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT date(date) AS granularity, COUNT(DISTINCT user_id) FROM click_log GROUP BY granularity");
    }

    @Test
    @DisplayName("Bounces")
    public void bouncesQueryTest() {
        qry = new QueryBuilder("Bounces", "Day", "");
        assertEquals(qry.getQuery(), "SELECT COUNT(id) FROM server_log WHERE (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 ");
        //assertEquals(qry.getQueryPerDay(), "SELECT distinct date(server_log.entry_date) AS uniq_date, COUNT(server_log.id) FROM server_log WHERE viewed = '1' OR (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 GROUP BY uniq_date");
        assertEquals(qry.getQuery("bounce rate"), "SELECT (CAST (COUNT(server_log.id) AS float)) / (SELECT (CAST (COUNT(click_log.id) AS float)) FROM click_log) FROM server_log WHERE (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 ");
    }

    @Test
    @DisplayName("Total Cost")
    public void totalCostQueryTest(){
        qry = new QueryBuilder("Total Cost", "Day", "");
        assertEquals(qry.getQuery(), "SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM impression_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT impression_tbl.granularity, impr_sum+click_sum FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(date) AS granularity, SUM(click_cost) AS click_sum FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity ");
    }

    @Test
    @DisplayName("Cost-per-click")
    public void CPCQueryTest(){
        qry = new QueryBuilder("Cost-Per-Click", "Day", "");
        assertEquals(qry.getQuery(), "SELECT (SUM(click_cost)+(SELECT SUM(impression_cost) FROM impression_log ))/ COUNT(click_log.id) FROM click_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT impression_tbl.granularity, (impr_sum+click_sum)/total_clicks FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(date) AS granularity, SUM(click_cost) AS click_sum, COUNT(click_log.id) as total_clicks FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity ");
    }

    @Test
    @DisplayName("Cost-per-acquisition")
    public void CPAQueryTest(){
        qry = new QueryBuilder("Cost-Per-Acquisition", "Day", "");
        assertEquals(qry.getQuery(), "SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log ");
        assertEquals(qry.getQueryPerDay(), "SELECT impression_tbl.granularity, impr_sum+click_sum / conv FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(date) AS granularity, SUM(click_cost) AS click_sum FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity INNER JOIN (SELECT date(entry_date) AS granularity, COUNT(id) AS conv FROM server_log WHERE conversion = 1 GROUP BY granularity) AS conv_tbl ON conv_tbl.granularity = click_tbl.granularity ");
    }

    @Test
    @DisplayName("Cost-Per-Thousand Impressions")
    public void CPMQueryTest(){
        qry = new QueryBuilder("Cost-Per-Thousand Impressions", "Day", "");
        assertEquals("SELECT 1000*(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ))/ COUNT(impression_log.id) FROM impression_log ",
                qry.getQuery());
        assertEquals("SELECT impression_tbl.granularity, 1000*(impr_sum+click_sum)/total_impressions FROM (SELECT date(impression_log.date) AS granularity, SUM(impression_cost) AS impr_sum, COUNT(id) as total_impressions FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(click_log.date) AS granularity, SUM(click_cost) AS click_sum FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity ",
                qry.getQueryPerDay());
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("imprCorrectQueries")
    @DisplayName("Impression - Filter")
    void impressionFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Impressions", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream imprCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0,"SELECT COUNT(id) FROM impression_log WHERE impression_log.age = '35-44' "),
                Arguments.of("income","High",0,"SELECT COUNT(id) FROM impression_log WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT COUNT(id) FROM impression_log WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT COUNT(id) FROM impression_log WHERE date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT COUNT(id) FROM impression_log WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1,"SELECT date(impression_log.date) AS granularity, COUNT(id) FROM impression_log WHERE impression_log.age = '35-44' GROUP BY granularity"),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT date(impression_log.date) AS granularity, COUNT(id) FROM impression_log WHERE date(impression_log.date) BETWEEN '2015-01-01' AND '2015-01-10' GROUP BY granularity")
        );
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("clickCorrectQueries")
    @DisplayName("Clicks - Filter")
    void clickFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Clicks", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream clickCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0,"SELECT COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' "),
                Arguments.of("income","High",0,"SELECT COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT COUNT(id) FROM click_log WHERE date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1,"SELECT date(impression_log.date) AS granularity, COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' GROUP BY granularity"),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT date(date) AS granularity, COUNT(id) FROM click_log WHERE date(date) BETWEEN '2015-01-01' AND '2015-01-10' GROUP BY granularity")
        );
    }

    @Test
    @DisplayName("Clicks - Filter Age Groups")
    void clickFilterAgesTest() {
        qry = new QueryBuilder("Clicks", "income", "High",0, "Day", "");
        assertEquals(qry.getQuery("age groups"), "SELECT DISTINCT age AS uniq_age, COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON impression_log.user_id = click_log.user_id WHERE impression_log.income = 'High' GROUP BY uniq_age");
        qry = new QueryBuilder("Clicks", "date", "2015-01-01,2015-01-10",0, "Day", "");
        assertEquals(qry.getQuery("age groups"), "SELECT DISTINCT age AS uniq_age, COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON impression_log.user_id = click_log.user_id WHERE click_log.date BETWEEN '2015-01-01' AND '2015-01-10' GROUP BY uniq_age");

    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("uniqCorrectQueries")
    @DisplayName("Unique Users - Filter")
    void uniqFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Unique Users", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream uniqCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0,"SELECT COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' "),
                Arguments.of("income", "High",0,"SELECT COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT COUNT(DISTINCT id) FROM click_log WHERE date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1,"SELECT date(impression_log.date) AS granularity, COUNT(DISTINCT click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' GROUP BY granularity"),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT date(click_log.date) AS granularity, COUNT(DISTINCT id) FROM click_log WHERE date(date) BETWEEN '2015-01-01' AND '2015-01-10' GROUP BY granularity")
        );
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("bouncesCorrectQueries")
    @DisplayName("Bounces - Filter")
    void bounceFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Bounces", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
   }

    private static Stream bouncesCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0, "SELECT COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44'  AND ((JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),
                Arguments.of("income", "High",0, "SELECT COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.income = 'High'  AND ((JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),
                Arguments.of("context","Shopping",0, "SELECT COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.context = 'Shopping'  AND ((JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE date BETWEEN '2015-01-01' AND '2015-01-10' AND ((JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),
                Arguments.of("gender","Male",0, "SELECT COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.gender = 'Male'  AND ((JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),
                Arguments.of("age","35-44",1, "SELECT date(impression_log.date) AS granularity, COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44'  AND (viewed = '1' OR (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30) GROUP BY granularity"),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT date(impression_log.date) AS granularity, COUNT(server_log.id) FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE date(impression_log.date) BETWEEN '2015-01-01' AND '2015-01-10' AND (viewed = '1' OR (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30) GROUP BY granularity")
        );
    }

    @Test
    @DisplayName("Bounce Rate - Filter")
    void bounceRateFilterTest() {
        qry = new QueryBuilder("Bounces", "age", "");
        assertEquals(qry.getQuery("bounce rate"), "SELECT (CAST (COUNT(server_log.id) AS float)) / (SELECT (CAST (COUNT(click_log.id) AS float)) FROM click_log) FROM server_log WHERE (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 ");
        qry = new QueryBuilder("Bounces", "income", "");
        assertEquals(qry.getQuery("bounce rate"), "SELECT (CAST (COUNT(server_log.id) AS float)) / (SELECT (CAST (COUNT(click_log.id) AS float)) FROM click_log) FROM server_log WHERE (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 ");
        qry = new QueryBuilder("Bounces", "date", "");
        assertEquals(qry.getQuery("bounce rate"), "SELECT (CAST (COUNT(server_log.id) AS float)) / (SELECT (CAST (COUNT(click_log.id) AS float)) FROM click_log) FROM server_log WHERE (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30 ");
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("totalCostCorrectQueries")
    @DisplayName("Total Cost - Filter")
    void totalCostFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Total Cost", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream totalCostCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0,"SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' "),
                Arguments.of("income", "High",0,"SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM impression_log WHERE date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1,"SELECT impression_tbl.granularity, impr_sum+click_sum FROM (SELECT date(impression_log.date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log WHERE age = '35-44' GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(click_log.date) AS granularity, SUM(click_cost) AS click_sum FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity "),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT impression_tbl.granularity, impr_sum+click_sum FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(date) AS granularity, SUM(click_cost) AS click_sum FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity WHERE impression_tbl.granularity BETWEEN '2015-01-01' AND '2015-01-10' ")
        );
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("CPACorrectQueries")
    @DisplayName("Cost-Per-Acquisition - Filter")
    void CPAFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Cost-Per-Acquisition", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream CPACorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44",0,"SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log WHERE impression_log.age = '35-44' "),
                Arguments.of("income", "High",0,"SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log WHERE date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT (ROUND(SUM(impression_cost)+(SELECT SUM(click_cost) FROM click_log ),2)) / (SELECT COUNT(id) FROM server_log WHERE conversion = 1 ) FROM impression_log WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1,"SELECT impression_tbl.granularity, impr_sum+click_sum / conv FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log WHERE age = '35-44' GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(click_log.date) AS granularity, SUM(click_cost) AS click_sum FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity INNER JOIN (SELECT date(entry_date) AS granularity, COUNT(server_log.id) AS conv FROM server_log INNER JOIN impression_log ON server_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' AND conversion = 1 GROUP BY granularity) AS conv_tbl ON conv_tbl.granularity = click_tbl.granularity "),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT impression_tbl.granularity, impr_sum+click_sum / conv FROM (SELECT date(date) AS granularity, SUM(impression_cost) AS impr_sum FROM impression_log GROUP BY granularity) AS impression_tbl INNER JOIN (SELECT date(date) AS granularity, SUM(click_cost) AS click_sum FROM click_log GROUP BY granularity) AS click_tbl ON impression_tbl.granularity = click_tbl.granularity INNER JOIN (SELECT date(entry_date) AS granularity, COUNT(id) AS conv FROM server_log WHERE conversion = 1 GROUP BY granularity) AS conv_tbl ON conv_tbl.granularity = click_tbl.granularity WHERE impression_tbl.granularity BETWEEN '2015-01-01' AND '2015-01-10' ")
        );
    }

    @ParameterizedTest(name = "{index} => Filter: {0} , Value: {1}, Grouped: {2}")
    @MethodSource("CPCCorrectQueries")
    @DisplayName("Cost-Per-Click - Filter")
    void CPCFilterTest(String filter , String filterVal, int grouped, String query) {
        qry = new QueryBuilder("Cost-Per-Click", filter, filterVal,grouped, "Day", "");
        assertEquals(qry.getQuery(), query);
    }

    private static Stream CPCCorrectQueries() {
        return Stream.of(
                Arguments.of("age","35-44", 0, "SELECT SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' "),
                Arguments.of("income", "High",0,"SELECT SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.income = 'High' "),
                Arguments.of("context","Shopping",0, "SELECT SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.context = 'Shopping' "),
                Arguments.of("date","2015-01-01,2015-01-10",0, "SELECT SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.date BETWEEN '2015-01-01' AND '2015-01-10' "),
                Arguments.of("gender","Male",0, "SELECT SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.gender = 'Male' "),
                Arguments.of("age","35-44",1, "SELECT date(impression_log.date) as granularity, SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON click_log.user_id = impression_log.user_id WHERE impression_log.age = '35-44' GROUP BY granularity"),
                Arguments.of("date","2015-01-01,2015-01-10",1, "SELECT date(impression_log.date) AS granularity, SUM(click_cost) / COUNT(click_log.id) FROM click_log INNER JOIN impression_log ON impression_log.user_id = click_log.user_id WHERE date(impression_log.date) BETWEEN '2015-01-01' AND '2015-01-10' GROUP BY granularity")
        );
    }
}
