package src;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.chart.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.text.TextAlignment;
import javafx.stage.Stage;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class DataViewScreen {

    private final Stage stage;
    private final DatabaseController dbc;
    private final QueryExecutor queryExecutor;
    private final FontSizeHandler fontSizeHandler;
    private final WindowsHandler windowsHandler;

    private Scene fileSelectScene;
    private final List<Button> metricButtons;
    private Filter currentFilter;
    private Button selectedButton;
    private ComboBox bounceRegistered;
    private VBox rightBorderPane;

    // DO NOT REMOVE WHITESPACE IN THESE STRINGS
    public static final String[] METRIC_BUTTON_TEXTS =
            { "Impressions",                   "Clicks",                 "Unique Users",
              "Bounces",                       "Conversions",            "Total Cost",
              "Click-Through-Rate",            "Cost-Per-Acquisition",   "Cost-Per-Click",
              "Cost-Per-Thousand Impressions", "Click Cost Distribution" };

    private final int METRIC_BUTTON_FONT_SIZE = 17;
    private final int COMBO_BOX_FONT_SIZE = 15;
    private final int METRIC_COUNT_FONT_SIZE = 19;
    private final int FILTERS_HEADING_FONT_SIZE = 19;
    private final int APPLIED_FILTERS_HEADING_FONT_SIZE = 17;
    private final int APPLIED_FILTER_FONT_SIZE = 15;
    private final int GRANULARITY_HEADING_FONT_SIZE = 17;

    public static final int CHART_TICK_FONT_SIZE = 12;
    public static final int AXIS_LABEL_FONT_SIZE = 13;
    public static final int CHART_TITLE_FONT_SIZE = 15;

    public DataViewScreen(Stage stage, DatabaseController dbc, QueryExecutor queryExecutor, FontSizeHandler fontSizeHandler, WindowsHandler windowsHandler, Scene fileSelectScene) {
        this.stage = stage;
        this.dbc = dbc;
        this.queryExecutor = queryExecutor;
        this.fontSizeHandler = fontSizeHandler;
        this.windowsHandler = windowsHandler;
        this.fileSelectScene = fileSelectScene;
        metricButtons  = new ArrayList<>();
    }

    private void configureStage(BorderPane borderPane, VBox leftVBox, VBox topVBox) {
        Scene scene = new Scene(borderPane);
        scene.heightProperty().addListener((observableValue, oldHeight, newHeight) -> sceneResizeEvent(newHeight.doubleValue(), leftVBox, topVBox));
        scene.getStylesheets().add("stylesheets/stylesheet.css");
        stage.setTitle("Data viewer");
        stage.setScene(scene);
        stage.setHeight(800);
        stage.setMinHeight(800);
        stage.setWidth(1400);
        stage.setMinWidth(1400);
        stage.show();
    }

    private void sceneResizeEvent(double newHeight, VBox leftVBox, VBox topVBox) {
        for (int i = 0; i < 10; i++) {
            Button button = (Button) leftVBox.getChildren().get(i);
            button.setPrefHeight((newHeight - topVBox.getPrefHeight()) / 11);
        }
    }

    public void showDataViewScreen() {
        BorderPane borderPane = new BorderPane();

        ComboBox<String> filtersComboBox = createComboBox(List.of("No Filter", "Date Range", "Gender", "Age", "Income", "Context"));
        filtersComboBox.getSelectionModel().select(0);
        ComboBox<String> granularityComboBox = createComboBox(List.of("Hour", "Day", "Week", "Month"));
        granularityComboBox.getSelectionModel().select(1);

        rightBorderPane = setupRightBorderPane(filtersComboBox, granularityComboBox);
        Label metricCount = (Label) rightBorderPane.getChildren().get(0);
        Label appliedFilterLabel = (Label) ((VBox) rightBorderPane.getChildren().get(1)).getChildren().get(3);

        VBox centreVBox = setupCentreBorderPane();
        VBox leftVBox = setupLeftBorderPane(filtersComboBox, granularityComboBox, centreVBox, metricCount);
        VBox topVBox = setupTopBorderPane();

        borderPane.setTop(topVBox);
        borderPane.setLeft(leftVBox);
        borderPane.setRight(rightBorderPane);
        borderPane.setCenter(centreVBox);

        filtersComboBox.valueProperty().addListener( ((observableValue, oldValue, newValue) -> {
            filterUpdate(newValue, metricCount, appliedFilterLabel);
            fontSizeHandler.updateFontSize(fontSizeHandler.DATA_VIEW_SCREEN);
        }));
        granularityComboBox.valueProperty().addListener(((observableValue, oldValue, newValue) -> {
            if (currentFilter != null) {       // if filter == null ==> "No Filter" is selected
                Platform.runLater(() -> {
                    currentFilter.filter();
                });
                filterUpdate(newValue, metricCount, appliedFilterLabel);
            }
            else {
                selectedButton.fire();
            }
        }));

        configureStage(borderPane, leftVBox, topVBox);

        Button impressionsButton = (Button) leftVBox.getChildren().get(0);
        impressionsButton.fire();
    }

    private VBox setupTopBorderPane() {
        VBox vbox = new VBox();
        MenuBarHandler menuBarHandler = new MenuBarHandler(stage, fontSizeHandler, windowsHandler);
        vbox.getChildren().add(menuBarHandler.dataViewScreen(dbc, fileSelectScene, queryExecutor));
        return vbox;
    }

    private VBox setupLeftBorderPane(ComboBox<String> filtersComboBox, ComboBox<String> granularityComboBox, VBox centreVBox, Label metricCount) {
        VBox leftButtonVBox = new VBox();

        MetricEventsHandler metricEventsHandler = new MetricEventsHandler(fontSizeHandler, queryExecutor, filtersComboBox, granularityComboBox, centreVBox, metricCount, this);

        for (String text : METRIC_BUTTON_TEXTS) {
            Button button = new Button(text);
            metricButtons.add(button);
            leftButtonVBox.getChildren().add(button);

            button.setPrefWidth(215);
            button.setPrefHeight(90);
            button.setTextAlignment(TextAlignment.CENTER);
            button.setStyle("-fx-background-radius: 0px; -fx-background-color: #e7e7e7;");
            button.setWrapText(true);

            button.setOnAction(e -> metricButtonClickEvent(button, metricEventsHandler));

            fontSizeHandler.addDVSNode(button, METRIC_BUTTON_FONT_SIZE);
        }

        return leftButtonVBox;
    }

    private void metricButtonClickEvent (Button button, MetricEventsHandler metricEventsHandler) {
        highlightSelectedButton(button);
        selectedButton = button;
        rightBorderPane.setDisable(false);
        metricEventsHandler.userClickEvent(button);
        fontSizeHandler.updateFontSize(fontSizeHandler.DATA_VIEW_SCREEN);
    }

    private void highlightSelectedButton(Button newSelected) {
        if (selectedButton != null) {
            selectedButton.setStyle(selectedButton.getStyle() + "; -fx-background-color: #e7e7e7;");
        }
        newSelected.setStyle(newSelected.getStyle() + "-fx-background-color: #c5c0c0;");
    }

    private VBox setupRightBorderPane(ComboBox<String> filterComboBox, ComboBox<String> granularityComboBox) {
        VBox vbox = new VBox();
        vbox.setPadding(new Insets(30,20,0,0));
        vbox.setAlignment(Pos.TOP_CENTER);
        vbox.setSpacing(80);

        Label metricCount = createNewLabel("");
        fontSizeHandler.addDVSNode(metricCount, METRIC_COUNT_FONT_SIZE);
        metricCount.setTextAlignment(TextAlignment.CENTER);

        VBox filtersVBox = setupFiltersVBox(filterComboBox);
        VBox granularityVBox = setupGranularityVBox(granularityComboBox);

        vbox.getChildren().addAll(metricCount, filtersVBox, granularityVBox);

        return vbox;
    }

    private VBox setupFiltersVBox(ComboBox<String> filterComboBox) {
        VBox vbox = new VBox();
        vbox.setSpacing(10);
        vbox.setAlignment(Pos.CENTER);

        Label filtersHeading = createNewLabel("Filters:");
        Label appliedFilterHeadingLabel = createNewLabel("Applied Filters:");
        Label appliedFilterLabel = createNewLabel("No Filter Applied");

        fontSizeHandler.addDVSNode(filtersHeading, FILTERS_HEADING_FONT_SIZE);
        fontSizeHandler.addDVSNode(appliedFilterHeadingLabel, APPLIED_FILTERS_HEADING_FONT_SIZE);
        fontSizeHandler.addDVSNode(appliedFilterLabel, APPLIED_FILTER_FONT_SIZE);

        vbox.getChildren().addAll(filtersHeading, filterComboBox, appliedFilterHeadingLabel, appliedFilterLabel);

        return vbox;
    }

    private VBox setupGranularityVBox(ComboBox<String> granularityComboBox) {
        VBox vbox = new VBox();
        vbox.setSpacing(10);
        vbox.setAlignment(Pos.CENTER);

        Label granularityLabel = createNewLabel("Granularity");
        vbox.getChildren().addAll(granularityLabel, granularityComboBox);
        fontSizeHandler.addDVSNode(granularityLabel, GRANULARITY_HEADING_FONT_SIZE);

        return vbox;
    }

    private VBox setupCentreBorderPane() {
        VBox chartsVBox = new VBox();
        chartsVBox.setSpacing(10);
        chartsVBox.setAlignment(Pos.CENTER);
        return chartsVBox;
    }

    private Label createNewLabel(String labelText) {
        return new Label(labelText);
    }

    private ComboBox<String> createComboBox(List<String> listItems) {
        ObservableList<String> observableList = FXCollections.observableList(listItems);
        ComboBox<String> comboBox = new ComboBox<>(observableList);
        fontSizeHandler.addDVSNode(comboBox, COMBO_BOX_FONT_SIZE);
        return comboBox;
    }

    // What to do when filters are updated
    private void filterUpdate(String newValue, Label metricCount, Label appliedFilters){
        if (newValue != null) {
            switch (newValue) {
                case "No Filter" -> {
                    Platform.runLater(() -> selectedButton.fire());
                    appliedFilters.setText("No filters applied.");
                    currentFilter = null;
                }
                case "Date Range" -> {
                    currentFilter = new DateFilter(queryExecutor, metricCount,this, selectedButton.getText(), fontSizeHandler);
                    currentFilter.load();
                }
                case "Age" -> {
                    String[] ages = {"<25", "25-34", "35-44", "45-54", ">54"};
                    currentFilter = new FilterController(queryExecutor, metricCount, ages, "Age",this, selectedButton.getText(), fontSizeHandler);
                    currentFilter.load();
                }
                case "Context" -> {
                    String[] contexts = {"News", "Shopping", "Social Media", "Blog", "Hobbies", "Travel"};
                    currentFilter = new FilterController(queryExecutor, metricCount, contexts, "Context", this, selectedButton.getText(), fontSizeHandler);
                    currentFilter.load();
                }
                case "Income" -> {
                    String[] incomes = {"High", "Medium", "Low"};
                    currentFilter = new FilterController(queryExecutor, metricCount, incomes, "Income", this, selectedButton.getText(), fontSizeHandler);
                    currentFilter.load();
                }
                case "Gender" -> {
                    String[] genders = {"Male", "Female"};
                    currentFilter = new FilterController(queryExecutor, metricCount, genders, "Gender", this, selectedButton.getText(), fontSizeHandler);
                    currentFilter.load();
                }
            }
        }
    }

    private void addToRightBorderPane(List<Node> ns){
        VBox bounceRegister = new VBox();
        bounceRegister.setSpacing(10);
        bounceRegister.setAlignment(Pos.CENTER);
        bounceRegister.getChildren().addAll(ns);
        rightBorderPane.getChildren().add(bounceRegister);
    }

    public LineChart getLineChart() {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox centreVBox = (VBox) borderPane.getCenter();
        return (LineChart) centreVBox.getChildren().get(0);
    }

    public PieChart getPieChart() {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox centreVBox = (VBox) borderPane.getCenter();
        return (PieChart) centreVBox.getChildren().get(1);
    }

    public String getGranularity() {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox rightVBox = (VBox) borderPane.getRight();
        VBox granularityVBox = (VBox) rightVBox.getChildren().get(2);
        ComboBox<String> comboBox = (ComboBox<String>) granularityVBox.getChildren().get(1);
        return comboBox.getSelectionModel().getSelectedItem();
    }

    public void setMultiChart(PieChart pieChart, LineChart lineChart) {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox centreVBox = (VBox) borderPane.getCenter();
        centreVBox.getChildren().clear();
        centreVBox.getChildren().addAll(lineChart, pieChart);
        centreVBox.setVgrow(lineChart, Priority.SOMETIMES);
        centreVBox.setVgrow(pieChart, Priority.SOMETIMES);
    }

    public void setLineChart(LineChart lineChart) {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox centreVBox = (VBox) borderPane.getCenter();
        centreVBox.getChildren().clear();
        centreVBox.getChildren().add(lineChart);
        centreVBox.setVgrow(lineChart, Priority.ALWAYS);
    }

    public void showFilterApplied(String text) {
        BorderPane borderPane = (BorderPane) stage.getScene().getRoot();
        VBox rightVBox = (VBox) borderPane.getRight();
        VBox filtersVBox = (VBox) rightVBox.getChildren().get(1);
        Label label = (Label) filtersVBox.getChildren().get(3);
        label.setText(text);
    }

    public void setBounceRegister(ComboBox box){
        this.bounceRegistered = box;
        Label register = new Label("Bounce Register:");
        fontSizeHandler.addDVSNode(bounceRegistered, COMBO_BOX_FONT_SIZE);
        fontSizeHandler.addDVSNode(register, COMBO_BOX_FONT_SIZE);
        ArrayList nodesToAdd = new ArrayList();
        nodesToAdd.add(register);
        nodesToAdd.add(bounceRegistered);
        addToRightBorderPane (nodesToAdd);
    }

    public String getBounceRegister() {
        return (bounceRegistered.getSelectionModel().getSelectedItem().toString());
    }

    public void removeBounceRegister() {
        Iterator it = rightBorderPane.getChildren().iterator();
        while (it.hasNext()){
            Node n = null;
            if ((n = (Node) it.next()).getClass() == VBox.class){
                boolean found = false;
                Iterator finder = ((VBox) n).getChildren().iterator();
                while (finder.hasNext()){
                    Node child = null;
                    if ((child = (Node) finder.next()).getClass() == Label.class){
                        if (((Label) child).getText().equals("Bounce Register:"))
                            found = true;
                    }
                }
                if (found)
                    it.remove();
            }
        }
    }

    public Filter getCurrentFilter() {
        return currentFilter;
    }

    public VBox getRightBorderPane() {return rightBorderPane;}
}