package src;

import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.stage.FileChooser;
import javafx.stage.Stage;

import java.io.File;
import java.nio.file.Paths;
import java.util.*;

public class FileSelectScreen {

    private final Stage stage;
    private final DatabaseController dbc;

    private Map<Node, Integer> fontSizeHashMap;

    private final MenuBarHandler menuBarHandler;
    private final FontSizeHandler fontSizeHandler;
    private final WindowsHandler windowsHandler;

    private int fileCount = 0;
    private final boolean[] logsGiven;
    private final String[] logsPath;

    private final int BUTTON_FONT_SIZE = 15;
    private final int LABEL_FONT_SIZE = 15;

    /*
    @desc - constructs the file select window
    @param - stage: stage to use as the window
    @param - db: database being used to store the data
    @param - dbc: controller for the database
    */
    public FileSelectScreen(Stage stage, DatabaseController dbc, FontSizeHandler fontSizeHandler, WindowsHandler windowsHandler) {
        this.stage = stage;
        this.dbc = dbc;

        this.fontSizeHandler = fontSizeHandler;
        this.windowsHandler = windowsHandler;
        fontSizeHashMap = new HashMap<>();

        menuBarHandler = new MenuBarHandler(stage, fontSizeHandler, windowsHandler);

        logsGiven = new boolean[3];
        logsPath = new String[3];
    }

    // sets up the GUI
    // configures the stage - width, height, title & scene
    public void setupInterface() {
        configureStage(setupScene(setupVBox()));
    }

    // sets up the scene from the StackPane
    private Scene setupScene(VBox vbox) {
        StackPane root = new StackPane();
        root.getChildren().add(vbox);
        return new Scene(root);
    }

    // configures the stage to have property values that are desirable and more user friendly
    // e.g. changes the height to an intuitive value
    private void configureStage(Scene scene) {
        fontSizeHandler.updateFontSize(fontSizeHandler.FILE_SELECT_SCREEN);
        int width = 550;
        int height = 350;

        scene.getStylesheets().add("stylesheets/stylesheet.css");
        stage.setMinWidth(width);
        stage.setMinHeight(height);
        stage.setWidth(width);
        stage.setHeight(height);
        stage.setTitle("File Selection");
        stage.setScene(scene);
        stage.show();
    }

    private VBox setupVBox() {
        VBox verticalVBox = new VBox(10);

        Label fileCountShowLabel = createNewLabel("0 files selected");
        Button continueButton = setupContinueButton();

        Label[] selectLabels = new Label[]{
                createNewLabel("No File Selected"),
                createNewLabel("No File Selected"),
                createNewLabel("No File Selected"),};

        Button[] removeButtons = new Button[] {
                setupRemoveButton(0, continueButton, selectLabels[0], fileCountShowLabel),
                setupRemoveButton(1, continueButton, selectLabels[1], fileCountShowLabel),
                setupRemoveButton(2, continueButton, selectLabels[2], fileCountShowLabel) };
        Button[] selectButtons = new Button[]{
                setupSelectButton(0, selectLabels[0], fileCountShowLabel, removeButtons[0], continueButton),
                setupSelectButton(1, selectLabels[1], fileCountShowLabel, removeButtons[1], continueButton),
                setupSelectButton(2, selectLabels[2], fileCountShowLabel, removeButtons[2], continueButton)};

        configureContinueButton(continueButton, selectButtons, removeButtons);
        HBox horizontalHBox = setupHorizontalHBox(continueButton, fileCountShowLabel);
        GridPane gridPane = setupGridPane(selectButtons, removeButtons, selectLabels);

        verticalVBox.getChildren().addAll(
                menuBarHandler.fileSelectScreen(),
                gridPane,
                horizontalHBox,
                setupCheatButton(continueButton));
        //menuBarHandler.updateFontSize("File Select Screen");

        return verticalVBox;
    }

    private Button setupCheatButton(Button continueButton) {
        // autoload files button
        Button cheatButton = new Button("Auto load files (GUI not updated, testing purposes)");
        cheatButton.setOnAction(e -> cheatButtonEvent(continueButton));
        fontSizeHandler.addFSSNode(cheatButton, BUTTON_FONT_SIZE);
        return cheatButton;
    }

    private void cheatButtonEvent(Button continueButton) {
        logsPath[0] = ".//impression_log.csv";
        logsPath[1] = ".//server_log.csv";
        logsPath[2] = ".//click_log.csv";
        continueButton.setDisable(false);
        logsGiven[0] = true;
        logsGiven[1] = true;
        logsGiven[2] = true;
        continueButton.fire();
    }

    private HBox setupHorizontalHBox(Button continueButton, Label fileCountShowLabel) {
        HBox horizontalHBox = new HBox(10);
        horizontalHBox.getChildren().addAll(fileCountShowLabel, continueButton);
        horizontalHBox.setAlignment(Pos.BASELINE_RIGHT);
        horizontalHBox.setPadding(new Insets(10));
        return horizontalHBox;
    }

    private Button setupContinueButton() {
        Button continueButton = new Button();
        fontSizeHandler.addFSSNode(continueButton, BUTTON_FONT_SIZE);
        return continueButton;
    }

    private void configureContinueButton(Button continueButton, Button[] selectLogButtons, Button[] removeLogButtons) {
        continueButton.setText("Continue");
        continueButton.setDisable(true);
        continueButton.setOnAction(e -> continueButtonEvent(continueButton, selectLogButtons, removeLogButtons));
        continueButton.setId("continue-btn");
    }

    private void continueButtonEvent(Button continueButton, Button[] selectLogButtons, Button[] removeLogButtons) {
        // store logs in database
        StackPane stackPane = (StackPane) stage.getScene().getRoot();
        ProgressIndicator progressIndicator = new ProgressIndicator();
        int piSize = 150;
        progressIndicator.setMinSize(piSize, piSize);
        progressIndicator.setPrefSize(piSize, piSize);
        stackPane.getChildren().add(progressIndicator);
        VBox vbox = (VBox) stackPane.getChildren().get(0);
        vbox.setDisable(true);

        Thread t = new Thread(
                () -> {
                    QueryExecutor queryExecutor = new QueryExecutor();
                    storeSelectedLogs();
                    Platform.runLater(() -> attemptGoDataViewScreen(continueButton, queryExecutor, stackPane));
                }
        );
        t.start();
    }

    private void attemptGoDataViewScreen(Button continueButton, QueryExecutor queryExecutor, StackPane stackPane) {
        Alert error = dbc.showAnyErrors();
        if (error != null) {
            error.show();
            continueButton.setText("Continue");
            stackPane.getChildren().remove(1);
            stackPane.getChildren().get(0).setDisable(false);
        }
        else {
            goToDataViewScreen(continueButton, queryExecutor);
        }
    }

    private void goToDataViewScreen(Button continueButton, QueryExecutor qry_exec) {
        continueButton.setText("Continue");
        // open the next screen
        stage.setMinWidth(1200);
        stage.setMinHeight(700);
        DataViewScreen dvs = new DataViewScreen(stage, dbc, qry_exec, fontSizeHandler, windowsHandler, stage.getScene());
        dvs.showDataViewScreen();
        fontSizeHandler.updateFontSize(fontSizeHandler.DATA_VIEW_SCREEN);
    }

    private void storeSelectedLogs() {
        if (logsGiven[0]) {
            dbc.storeImpressionLog(logsPath[0]);
        }
        if (logsGiven[1]) {
            dbc.storeServerLog(logsPath[1]);
        }
        if (logsGiven[2]) {
            dbc.storeClickLog(logsPath[2]);
        }
    }

    // sets up the GridPane with valid properties
    private GridPane setupGridPane(Button[] selectButtons, Button[] removeButtons, Label[] selectLabels) {
        GridPane selectionGridPane = new GridPane();
        selectionGridPane.setPadding(new Insets(10,20,20,20));
        selectionGridPane.setHgap(20);
        selectionGridPane.setVgap(20);

        // labels showing which log is which
        selectionGridPane.add(createNewLabel("Impression Log"), 0, 0);
        selectionGridPane.add(createNewLabel("Server Log"), 0, 1);
        selectionGridPane.add(createNewLabel("Click Log"), 0, 2);
        // buttons which allow the user to select the logs
        selectionGridPane.add(selectButtons[0], 1, 0);
        selectionGridPane.add(selectButtons[1], 1, 1);
        selectionGridPane.add(selectButtons[2], 1, 2);
        // buttons which allow the user to remove the selected logs
        selectionGridPane.add(removeButtons[0], 2, 0);
        selectionGridPane.add(removeButtons[1], 2, 1);
        selectionGridPane.add(removeButtons[2], 2, 2);
        // labels which say whether a file has been selected, and if one has then it shows the name of this file
        selectionGridPane.add(selectLabels[0], 3, 0);
        selectionGridPane.add(selectLabels[1], 3, 1);
        selectionGridPane.add(selectLabels[2], 3, 2);

        return selectionGridPane;
    }

    private Label createNewLabel(String labelText) {
        Label label = new Label(labelText);
        fontSizeHandler.addFSSNode(label, LABEL_FONT_SIZE);
        return label;
    }

    ////////////////////
    private Button setupRemoveButton(int log, Button continueButton, Label selectedLabel, Label fileCountShowerLabel) {
        Button removeButton = new Button("Remove File");
        removeButton.setDisable(true);
        removeButton.setOnAction(e -> removeLogEvent(log,removeButton, continueButton, selectedLabel, fileCountShowerLabel));
        fontSizeHandler.addFSSNode(removeButton, BUTTON_FONT_SIZE);
        return removeButton;
    }

    ////////////////////
    private void removeLogEvent(int log, Button removeImpressionLogButton, Button continueButton, Label impressionSelectedLabel, Label fileCountShowerLabel) {
        removeImpressionLogButton.setDisable(true);
        logsGiven[log] = false;
        logsPath[log] = null;
        impressionSelectedLabel.setText("No File Selected");
        fileCount--;
        fileCountShowerLabel.setText(fileCount + " file(s) selected");
        buttonEnabler(continueButton);
    }

    ////////////////////
    private Button setupSelectButton(int log, Label selectLabel, Label fileCountShowerLabel, Button removeButton, Button continueButton) {
        Button selectButton = new Button("Select File");
        selectButton.setOnAction(e -> selectLogEvent(log,selectLabel, fileCountShowerLabel, removeButton, continueButton));
        fontSizeHandler.addFSSNode(selectButton, BUTTON_FONT_SIZE);
        return selectButton;
    }

    private void selectLogEvent(int log, Label selectLabel, Label fileCountShowerLabel, Button removeButton, Button continueButton) {
        File f;
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Open File to Load");

        String currentPath = Paths.get(".").toAbsolutePath().normalize().toString();
        fileChooser.setInitialDirectory(new File(currentPath));

        FileChooser.ExtensionFilter csvFilter = new FileChooser.ExtensionFilter("Text files", "*.csv");
        fileChooser.getExtensionFilters().add(csvFilter);
        f = fileChooser.showOpenDialog(stage);
        logsGiven[log] = true;
        if (selectLabel.getText().equals("No File Selected")) {
            selectLabel.setText(f.getName());
            fileCount++;
            fileCountShowerLabel.setText(fileCount + " file(s) selected");
        }
        else {
            selectLabel.setText(f.getName());
        }
        removeButton.setDisable(false);
        buttonEnabler(continueButton);
        //store click log path
        logsPath[log] = f.getAbsolutePath();
    }
    ////////////////////

    /*
    @desc - Disables/Enables list of buttons
    @param - set - Boolean value to set disable to
    @param - buttons - ArrayList of buttons to set to disabled/not disabled
    */
    private void setDisable(boolean set, ArrayList<Button> buttons) {
        buttons.stream().parallel().forEach(b -> b.setDisable(set));
    }

    private void buttonEnabler(Button buttonEnabler) {
        if (logsGiven[0]) {
            buttonEnabler.setDisable(!logsGiven[2] && logsGiven[1]);
        }
        else {
            buttonEnabler.setDisable(true);
        }
    }
}