package src;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.chart.*;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.List;

public class FilterController extends Filter {

    private QueryExecutor qry_exec;
    private String selectedValue;
    private Label metric;
    private String[] filterValues;
    private String filterName;
    private DataViewScreen dvs;
    private String metricName;
    private FontSizeHandler fontSizeHandler;

    private final int BUTTONS_FONT_SIZE = 16;
    private final int CHART_TICK_FONT_SIZE = 10;
    private final int AXIS_LABEL_FONT_SIZE = 12;
    private final int LABEL_FONT_SIZE = 18;
    /*
    @param db_ctrl - The controller for the database
    @param metric - a label that displays the total count for the given metric
    @param filterValues - an array containing all of the values that can be selected for that filter e.g. gender filter would be { "Male", "Female" }
    @param filterName - the name of the filter e.g. "Gender", "Income" etc.
    @param tableName - the name of the table data will be queried from
    @param columns - the return columns for the db query
    @param dvs - the object handling the main screen that is navigated to after the filter is applied or cancelled
    @param metricName - the name of the metric to be filtered
    @param fontSize - size of font to use
     */
    public FilterController(QueryExecutor qry_exec, Label metric, String[] filterValues, String filterName, DataViewScreen dvs, String metricName, FontSizeHandler fontSizeHandler) {
        this.qry_exec = qry_exec;
        this.metric = metric;
        this.filterValues = filterValues;
        this.filterName = filterName;
        this.dvs = dvs;
        this.metricName = metricName;
        this.fontSizeHandler = fontSizeHandler;
    }

    @Override
    public void load() {
        Stage filterStage = new Stage();

        BorderPane borderPane = new BorderPane();
        VBox filterOptions = new VBox();
        HBox controls = new HBox();
        HBox title = new HBox();

        fontSizeHandler.resetFPUNodes();

        Button[] dataButtons = new Button[filterValues.length];
        for (int i = 0; i < dataButtons.length; i++) {
            Button filterButton = new Button(filterValues[i]);
            dataButtons[i] = filterButton;
            fontSizeHandler.addFPUNode(filterButton, BUTTONS_FONT_SIZE);
            filterOptions.getChildren().add(filterButton);
            filterButton.setOnAction(
                    action -> {
                        setFilterValues(new String[] { filterButton.getText() });
                        filterStage.setScene(super.done());
                        try {
                            java.util.concurrent.TimeUnit.SECONDS.sleep(1);
                        } catch (InterruptedException e) {
                            System.err.println("Interrupted exception: " + e);
                        }
                        filterStage.close();
                        filter();
                    }
            );
        }

        Button cancel = new Button ("Cancel");
        cancel.setOnAction(
                action -> {
                    tearDown();
                    filterStage.close();
                }
        );
        cancel.setPadding(new Insets(10));
        fontSizeHandler.addFPUNode(cancel, BUTTONS_FONT_SIZE);

        filterOptions.setAlignment(Pos.CENTER);
        filterOptions.setSpacing(10);

        controls.setAlignment(Pos.CENTER);
        controls.setPadding(new Insets(5));
        controls.getChildren().add(cancel);

        title.setAlignment(Pos.CENTER);
        Label top = new Label("Select " + filterName);
        top.setPadding(new Insets(10));
        fontSizeHandler.addFPUNode(top, LABEL_FONT_SIZE);
        title.getChildren().add(top);

        borderPane.setBottom(controls);
        borderPane.setCenter(filterOptions);
        borderPane.setTop(title);

        fontSizeHandler.updateFontSize(fontSizeHandler.FILTER_POPUP);

        filterStage.setTitle(filterName + " Picker");
        filterStage.setScene(new Scene(borderPane));
        filterStage.setMinHeight(300);
        filterStage.setMinWidth(300);
        filterStage.sizeToScene();
        filterStage.show();
    }

    @Override
    public void filter() {
        // Update Label
        String currentVal = metric.getText();
        String[] valArray = currentVal.split(" ");
        String granularity = dvs.getGranularity();
        String option = "";
        System.out.println(currentVal);
        if (currentVal.contains("Bounces"))
            option = dvs.getBounceRegister();

        List<List<String[]>> queryResponse = qry_exec.executeQuery(metricName,filterName.toLowerCase(),selectedValue,granularity, option);
        String filteredValue = queryResponse.get(0).get(0)[0];
        if (filteredValue == null)
            filteredValue = "0";

        int replace = 0;
        if (!currentVal.contains("Bounces")) {
            for (String val : valArray) {
                if (val.contains(":")) {
                    break;
                }
                replace++;
            }
        } else {
            replace = valArray.length -1;
            for (int i = valArray.length-1 ; i >=0; i--){
                if (valArray[i].contains(":")) {
                    break;
                }
                replace--;
            }
        }
        String updatedMetric = currentVal.replace(valArray[replace+1], filteredValue);
        metric.setText(updatedMetric);


        List<String[]> filteredValues = queryResponse.get(1);
        if (metricName.equals("Bounces")) {
            PieChart pieChart = updateBounces(Integer.valueOf(filteredValue));
            LineChart lineChart = updateBounceRate(filteredValues, granularity, option);
            lineChart.getXAxis().tickLabelFontProperty().set(Font.font(CHART_TICK_FONT_SIZE + (fontSizeHandler.getSizeValue() * 5)));
            dvs.setMultiChart(pieChart, lineChart);
        } else {
            XYChart.Series data = new XYChart.Series();
            data.setName(dvs.getLineChart().getTitle());


            //cumulative chart for total cost, standard for others
            if (metricName.equals("Total Cost")) {
                double cumulativeCost = 0;
                for (int i = 0; i < filteredValues.size(); i++) {
                    String[] record = filteredValues.get(i);
                    cumulativeCost = cumulativeCost + Double.parseDouble(record[1]);
                    data.getData().add(new XYChart.Data(record[0], cumulativeCost));
                }

            } else {
                for (int i = 0; i < filteredValues.size(); i++) {
                    String[] record = filteredValues.get(i);
                    data.getData().add(new XYChart.Data(record[0], Double.valueOf(record[1])));
                }
            }

            Axis xAxis = new CategoryAxis();
            NumberAxis yAxis = new NumberAxis();
            yAxis.setLabel(dvs.getLineChart().getYAxis().getLabel());
            xAxis.setLabel("Day");
            LineChart lineChart = new LineChart(xAxis,yAxis);
            lineChart.getData().addAll(data);
            lineChart.setTitle(dvs.getLineChart().getTitle());
            if (metricName.equals("Clicks")){
                PieChart pieChart = dvs.getPieChart();
                pieChart.getData().clear();

                List<String[]> ageValues = qry_exec.executeQuery("Clicks", filterName.toLowerCase(), selectedValue, "age groups",2, "");
                for(String[] record : ageValues) {
                    pieChart.getData().add(new PieChart.Data(record[0], Integer.valueOf(record[1])));
                }
                dvs.setMultiChart(pieChart, lineChart);
            }
            else {
                dvs.setLineChart(lineChart);
            }
            lineChart.setStyle("; -fx-font-size: " + (DataViewScreen.CHART_TITLE_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())) + ";");
            lineChart.getXAxis().tickLabelFontProperty().set(Font.font(CHART_TICK_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
            lineChart.getYAxis().tickLabelFontProperty().set(Font.font(CHART_TICK_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
            yAxis.setTickLabelFont(Font.font(AXIS_LABEL_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
            xAxis.setTickLabelFont(Font.font(AXIS_LABEL_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
        }
        dvs.showFilterApplied(filterName + " Filter: " + selectedValue);
    }

    private PieChart updateBounces(Integer bounces) {
        // Update pie chart
        PieChart pieChart = dvs.getPieChart();
        pieChart.getData().clear();
        Integer totalClicks = Integer.valueOf(qry_exec.executeQuery("Clicks", "Day", "").get(0).get(0)[0]);
        pieChart.getData().add(new PieChart.Data("Normal clicks",totalClicks-bounces));
        pieChart.getData().add(new PieChart.Data("Bounce clicks", bounces));

        return pieChart;
    }

    private LineChart updateBounceRate(List<String[]> filteredValues, String granularity, String option){

        //String bounceRate = db_ctrl.dbQuery(SQLCommands.filter("server_log",filterName.toLowerCase(),selectedValue, new String[]{"(CAST (COUNT(server_log.id) AS float)) / (SELECT (CAST (COUNT(click_log.id) AS float)) FROM click_log)"}, " AND (viewed = '1' OR (JULIANDAY(exit_date) - JULIANDAY(entry_date)) * 60 * 60 * 24 < 30)"),1).get(0)[0];
        String bounceRate = qry_exec.executeQuery(metricName,filterName.toLowerCase(), selectedValue,"bounce rate",1, option).get(0)[0];
        //bounceRateLabel.setText("Bounce rate: "+bounceRate);

        //List<String[]> clicksPerDay = db_ctrl.dbQuery(SQLCommands.clickLogDates, 2);
        List<String[]> clicksPerDay = qry_exec.executeQuery("Clicks", granularity, "").get(1);
        XYChart.Series metric = new XYChart.Series();
        metric.setName("Bounce rate per day");

        int i =0;
        while(i<filteredValues.size()){
            double avg = (Double.parseDouble(filteredValues.get(i)[1])) / (Double.parseDouble(clicksPerDay.get(i)[1]));
            metric.getData().add(new XYChart.Data(filteredValues.get(i)[0], avg));
            i++;

        }
        ArrayList<XYChart.Series> data = new ArrayList<XYChart.Series>(){
            {
                add(metric);
            }
        };

        Axis x = new CategoryAxis();
        x.setLabel("Day");
        Axis y = new NumberAxis();
        y.setLabel("Bounce Rate");

        LineChart linechart = new LineChart(x, y);
        linechart.getData().addAll(data);
        linechart.setTitle("Bounce rate per day");
        return linechart;
    }

    @Override
    public void tearDown() {
        this.selectedValue = null;
    }

    @Override
    public void setFilterValues(String[] filterValues) {
        selectedValue = filterValues[0];
    }

    @Override
    public String getFilterName() {
        return filterName;
    }

    public String getSelectedValue() {
        return selectedValue;
    }
}
